﻿using System.Windows.Forms;
using XEngine;

namespace X_Editor
{
    public class XHeightMap_Plugin : ComponentPlugin
    {
        public XHeightMap_Plugin(XMain X) : base(X)
        {
            type = typeof(XHeightMap);
            Name = "Height Map Terrain";
        }

        public override ListViewItem SetupListViewItem()
        {
            ListViewItem item = new ListViewItem();
            item.Text = Name;

            XHeightMap heightmap = new XHeightMap(X, null, null, null, null, null, null);
            item.Tag = heightmap;

            return item;
        }

        public override void UpdateObjectProperties(object Input, PropertyGrid Properties, ListView Scene)
        {
            XHeightMap heightmap = (XHeightMap)Input;

            if (!string.IsNullOrEmpty(heightmap.HeightMapFile) && !string.IsNullOrEmpty(heightmap.TextureMapFile) && heightmap.Params != null)
            {
                try
                {
                    heightmap.Load(X.game.Content);

                    if (Properties != null && Properties.SelectedObject == heightmap)
                        Properties.SelectedObject = heightmap;
                }
                catch
                {
                    MessageBox.Show("There was a problem loading one of the heightmap's files. Check that the file exists and has been built with the Texture importer and processor.", "File Error");
                }
            }
        }

        public override void AcceptDragDrop(object Input, object DraggedItem, PropertyGrid Properties, ListView Scene)
        {
            XHeightMap heightmap = (XHeightMap)Input;

            if (Tools.GetPropertyAtPoint(Properties.PointToClient(Cursor.Position), Properties).Label == "Params" && DraggedItem is XEnvironmentParameters)
                heightmap.Params = (XEnvironmentParameters)DraggedItem;
            else if (Tools.GetPropertyAtPoint(Properties.PointToClient(Cursor.Position), Properties).Label == "HeightMapFile" && DraggedItem is ContentItem)
                heightmap.HeightMapFile = ((ContentItem)DraggedItem).GenerateFilename().Replace("\\Content", "Content");
            else if (Tools.GetPropertyAtPoint(Properties.PointToClient(Cursor.Position), Properties).Label == "RTextureFile" && DraggedItem is ContentItem)
                heightmap.RTextureFile = ((ContentItem)DraggedItem).GenerateFilename().Replace("\\Content", "Content");
            else if (Tools.GetPropertyAtPoint(Properties.PointToClient(Cursor.Position), Properties).Label == "GTextureFile" && DraggedItem is ContentItem)
                heightmap.GTextureFile = ((ContentItem)DraggedItem).GenerateFilename().Replace("\\Content", "Content");
            else if (Tools.GetPropertyAtPoint(Properties.PointToClient(Cursor.Position), Properties).Label == "BTextureFile" && DraggedItem is ContentItem)
                heightmap.BTextureFile = ((ContentItem)DraggedItem).GenerateFilename().Replace("\\Content", "Content");
            else if (Tools.GetPropertyAtPoint(Properties.PointToClient(Cursor.Position), Properties).Label == "TextureMapFile" && DraggedItem is ContentItem)
                heightmap.TextureMapFile = ((ContentItem)DraggedItem).GenerateFilename().Replace("\\Content", "Content");
            
            Properties.SelectedObject = heightmap;
            UpdateObjectProperties(heightmap, Properties, Scene);
        }

        public override void WriteToXML(System.Xml.XmlWriter writer, object obj)
        {
            XHeightMap heightmap = (XHeightMap)obj;

            writer.WriteStartElement("sceneitem");
            writer.WriteAttributeString("Type", Name);
            writer.WriteAttributeString("HeightMap", heightmap.HeightMapFile);
            writer.WriteAttributeString("TextureMap", heightmap.TextureMapFile);
            writer.WriteAttributeString("RTexture", heightmap.RTextureFile);
            writer.WriteAttributeString("GTexture", heightmap.GTextureFile);
            writer.WriteAttributeString("BTexture", heightmap.BTextureFile);
            writer.WriteAttributeString("AutoDraw", heightmap.AutoDraw.ToString());
            writer.WriteAttributeString("DrawOrder", heightmap.DrawOrder.ToString());
            writer.WriteAttributeString("EnvParamsNum", heightmap.environmentalParametersNumber.ToString());
            writer.WriteEndElement();
        }

        public override void LoadFromXML(System.Xml.XmlNode node, ListView scene)
        {
            ListViewItem sceneitem = SetupListViewItem();

            XHeightMap heightmap = new XHeightMap(X, node.Attributes["HeightMap"].InnerText, null, node.Attributes["RTexture"].InnerText, node.Attributes["GTexture"].InnerText, node.Attributes["BTexture"].InnerText, node.Attributes["TextureMap"].InnerText);
            heightmap.AutoDraw = bool.Parse(node.Attributes["AutoDraw"].InnerText);
            heightmap.DrawOrder = int.Parse(node.Attributes["DrawOrder"].InnerText);

            int paramsNum = int.Parse(node.Attributes["EnvParamsNum"].InnerText);
            heightmap.environmentalParametersNumber = paramsNum;


            foreach (ListViewItem item in scene.Items)
                if (item.Tag is XEnvironmentParameters)
                    if (((XEnvironmentParameters)item.Tag).number == paramsNum)
                        heightmap.Params = (XEnvironmentParameters)item.Tag;

            if (!string.IsNullOrEmpty(heightmap.TextureMapFile) && !string.IsNullOrEmpty(heightmap.HeightMapFile) && heightmap.Params != null)
                heightmap.Load(X.game.Content);

            sceneitem.Text = Name;
            sceneitem.Tag = heightmap;
            sceneitem.Group = scene.Groups["Environment"];

            scene.Items.Add(sceneitem);
        }
    }
}
